-- debug_status = 1
debug_mod_name = "ProgressiveRunning"
debug_file = debug_mod_name .. "-debug.txt"
require("utils")
require("config")

--------------------------------------------------------------------------------------
function init_globals()
	-- initialize or update general globals of the mod
	debug_print( "init_globals " )
	
	global.ticks = global.ticks or 0
	global.player_mem = global.player_mem or {}
end

--------------------------------------------------------------------------------------
local function init_player(player)
	if global.player_mem == nil then return end
	
	-- initialize or update per player globals of the mod, and reset the gui
	debug_print( "init_player ", player.name, " connected=", player.connected )
	
	global.player_mem[player.index] = global.player_mem[player.index] or { player = player }
	
	local player_mem = global.player_mem[player.index]
	player_mem.walking = true -- to trigger the initialisation of player_mem.modif at first tick
	player_mem.toggle = 1 -- 0 slow, 1 prog, 2 fast
	player_mem.modif = 0 -- current modifier
	player_mem.modif_max = 0 -- record the modifier maximum (with all speed gear working)
	player_mem.acceleration = false
end

--------------------------------------------------------------------------------------
local function init_players()
	for _, player in pairs(game.players) do
		init_player(player)
	end
end

--------------------------------------------------------------------------------------
local function on_init() 
	-- called once, the first time the mod is loaded on a game (new or existing game)
	debug_print( "on_init" )
	init_globals()
	init_players()
end

script.on_init(on_init)

--------------------------------------------------------------------------------------
local function on_configuration_changed(data)
	-- detect any mod or game version change
	if data.mod_changes ~= nil then
		local changes = data.mod_changes[debug_mod_name]
		if changes ~= nil then
			debug_print( "update mod: ", debug_mod_name, " ", tostring(changes.old_version), " to ", tostring(changes.new_version) )
		
			init_globals()
			init_players()
			
			if changes.old_version and older_version(changes.old_version , "1.0.15") then
				message_all( "Progressive running : now you can use F1 hotkey to rotate between slow > progressive > fast !" )
			end
		end	
	end
end

script.on_configuration_changed(on_configuration_changed)

--------------------------------------------------------------------------------------
local function on_player_created(event)
	-- called at player creation
	local player = game.players[event.player_index]
	debug_print( "player created ", player.name )
	
	init_player(player)
end

script.on_event(defines.events.on_player_created, on_player_created )

--------------------------------------------------------------------------------------
local function on_player_joined_game(event)
	-- called in SP(once) and MP(every connect), eventually after on_player_created
	local player = game.players[event.player_index]
	debug_print( "player joined ", player.name )
	
	init_player(player)
end

script.on_event(defines.events.on_player_joined_game, on_player_joined_game )

--------------------------------------------------------------------------------------
local function on_player_left_game(event)
	local player = game.players[event.player_index]
	debug_print( "player left ", player.name )
	
	-- do not do this : when lefting, character does not exits anymore.
	-- player.character_running_speed_modifier = 0 -- in case of uninstall, to try to retrieve initial speed modifier
end

script.on_event(defines.events.on_player_left_game, on_player_left_game )

--------------------------------------------------------------------------------------
local function on_player_driving_changed_state(event)
	local player = game.players[event.player_index]
	debug_print( "on_player_driving_changed_state ", player.name )
end

-- script.on_event(defines.events.on_player_driving_changed_state, on_player_driving_changed_state )

--------------------------------------------------------------------------------------
local function get_speed_modifier(player)
	local inv_armor = player.get_inventory(defines.inventory.player_armor)
	local modif = 0
	
	if inv_armor then
		local armor = inv_armor[1] 
		if armor.valid_for_read and armor.grid then
			local equips = armor.grid.equipment
			for _, equip in pairs(equips) do
				-- debug_print( "equip ", equip.name, " ", equip.movement_bonus )
				modif = modif + equip.movement_bonus
			end
		end
	end
	
	return(modif)
end

local function format_speed(m)
	return( string.format("%.3g",m) )
end

--------------------------------------------------------------------------------------
local function on_progrun_hotkey(event)
	local player = game.players[event.player_index]
	local player_mem = global.player_mem[player.index]
	local s = ""
	
	player_mem.toggle = player_mem.toggle + 1
	if player_mem.toggle == 3 then player_mem.toggle = 0 end
	
	if player_mem.toggle == 0 then
		s = "-"
	elseif player_mem.toggle == 1 then
		s = "<"
	elseif player_mem.toggle == 2 then
		s = "+"
	end

	player.surface.create_entity({name = "flying-text", position = pos_offset(player.position,{x=-0.5,y=0.2}), text = s, color = {r=1, g=1, b=0}})
	player_mem.walking = true
end

script.on_event("progrun_hotkey", on_progrun_hotkey)

--------------------------------------------------------------------------------------
local function on_tick(event)
	-- player.character_running_speed_modifier is 0 in normal.
	-- if > 1 speed is higher, 1>0 speed reduce, <0 backward.
	
	if global.ticks <= 0 then 
		global.ticks = 15
		
		for _, player in pairs(game.players) do
			if player.connected and player.character ~= nil then
				local player_mem = global.player_mem[player.index]
				local walking = player.walking_state.walking
				
				if walking then
					if player_mem.toggle == 2 then
						-- fast
						player_mem.modif = 0
						player_mem.acceleration = false
						player_mem.walking = true
						player.character_running_speed_modifier = player_mem.modif
					else
						if not player_mem.walking then 
							-- start slow
							player_mem.modif_max = get_speed_modifier(player)
							-- debug_print("max=", player_mem.modif_max)
							player_mem.acceleration = true
							player_mem.modif = initial_speed_factor/(1+player_mem.modif_max) - 1 
							player_mem.walking = true
							player.character_running_speed_modifier = player_mem.modif
						else 
							if player_mem.acceleration and player_mem.toggle == 1 then
								-- accelerate
								player_mem.modif = player_mem.modif +0.15*acceleration_factor
								if player_mem.modif > -0.05 then
									player_mem.modif = 0
									player_mem.acceleration = false
								end
								player.character_running_speed_modifier = player_mem.modif
							end
						end
					end
				else
					if player_mem.walking then
						if player_mem.toggle < 2 then
							-- reset to slow walking
							player_mem.modif_max = get_speed_modifier(player)
							player_mem.modif = initial_speed_factor/(1+player_mem.modif_max) -1
							player_mem.walking = false
							player.character_running_speed_modifier = player_mem.modif
						end
					end
				end
				
				-- debug_print( "speed ", player.name,
					-- " cur=", format_speed(player.character_running_speed_modifier), 
					-- " cur=", format_speed(player_mem.modif), 
					-- " max=", format_speed(player_mem.modif_max),
					-- " acc=", player_mem.acceleration )
			end
		end
	else
		global.ticks = global.ticks -1 
	end
end

script.on_event(defines.events.on_tick, on_tick)

--------------------------------------------------------------------------------------

local interface = {}

function interface.on()
	debug_print( "on" )
	
	script.on_event(defines.events.on_tick, on_tick)
end

function interface.off()
	debug_print( "off" )
	
	for _, player in pairs(game.players) do
		player.character_running_speed_modifier = 0
	end
	
	script.on_event(defines.events.on_tick, nil)
end

remote.add_interface( "progrun", interface )

-- /c remote.call( "progrun", "on" )
-- /c remote.call( "progrun", "off" )